﻿using ShutdownTimer.Core.Helpers;
using ShutdownTimer.Services;

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using System.Timers;
using Windows.ApplicationModel;
using Windows.Foundation.Metadata;
using Windows.System;
using Windows.UI.Core;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace ShutdownTimer.Views
{
    public sealed partial class WatchTimerPage : Page, INotifyPropertyChanged
    {
        private System.Timers.Timer timerClock;

        public WatchTimerPage()
        {
            InitializeComponent();

            timerClock = new System.Timers.Timer(1000);
            timerClock.Elapsed += TimerClock_Tick;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void Set<T>(ref T storage, T value, [CallerMemberName]string propertyName = null)
        {
            if (Equals(storage, value))
            {
                return;
            }

            storage = value;
            OnPropertyChanged(propertyName);
        }

        private void OnPropertyChanged(string propertyName) => PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            UpdateUI();

            switch (Clockwork.GetTimer().action)
            {
                case "shutdown":
                    CountdownIcon.Glyph = "\xE7E8";
                    break;

                case "hibernate":
                    CountdownIcon.Glyph = "\xE823";
                    break;

                case "sleep":
                    CountdownIcon.Glyph = "\xE708";
                    break;

                case "lock":
                    CountdownIcon.Glyph = "\xE72E";
                    break;

                case "logout":
                    CountdownIcon.Glyph = "\xF3B1";
                    break;

                case "restart":
                    CountdownIcon.Glyph = "\xE72C";
                    break;
            }

            if (!Clockwork.IsZero() && timerClock.Enabled == false)
            {
                timerClock.Start();
                ShowCancel(true);
                ShowEdit(true);
                ShowMinimize(true);
            }
        }

        private void TimerClock_Tick(Object source, ElapsedEventArgs e)
        {
            Core.Models.Timer timer = Clockwork.GetTimer();

            if (timer.hours == 0 && timer.minutes == 5 && timer.seconds == 0)
            {
                Services.ToastNotificationsService toastNotificationsService = new Services.ToastNotificationsService();
                toastNotificationsService.ShowTimeLeftNotification(timer.action, GetTimeFormatted(timer, true));
            }

            if (!Clockwork.IsZero())
            {
                Clockwork.CountDown();
            }
            else
            {
                timerClock.Stop();
                ExitWindowsAsync();
            }

            UpdateUI();
        }

        private void InitializeClock(int hours, int minutes, int seconds)
        {
            Clockwork.UpdateTime(hours: hours, minutes: minutes, seconds: seconds, currentTime: true, initialTime: true);
        }

        private string GetTimeFormatted(Core.Models.Timer timer, bool UseInitialTime = false)
        {
            string hours, minutes, seconds;

            if (!UseInitialTime)
            {
                hours = Numeric.AddZeros(timer.hours);
                minutes = Numeric.AddZeros(timer.minutes);
                seconds = Numeric.AddZeros(timer.seconds);
            }
            else
            {
                hours = Numeric.AddZeros(timer.initialHours);
                minutes = Numeric.AddZeros(timer.initialMinutes);
                seconds = Numeric.AddZeros(timer.initialSeconds);
            }


            string output = hours + ":" + minutes + ":" + seconds;
            return output;
        }

        private async void UpdateUI()
        {
            Core.Models.Timer timer = Clockwork.GetTimer();

            string timeFormatted = GetTimeFormatted(timer);

            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                CountdownClock.Text = timeFormatted;
            });
        }

        private async void ShowCancel(bool show)
        {
            if (show)
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    CancelButton.IsEnabled = true;
                });
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    CancelButton.IsEnabled = false;
                });
            }
        }

        private async void ShowEdit(bool show)
        {
            if (show)
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    EditButton.IsEnabled = true;
                });
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    EditButton.IsEnabled = false;
                });
            }
        }

        private async void ShowMinimize(bool show)
        {
            if (show)
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    MinimizeButton.IsEnabled = true;
                });
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    MinimizeButton.IsEnabled = false;
                });
            }
        }

        /// <summary>
        /// Runs the choosen timer action.
        /// </summary>
        private async Task ExitWindowsAsync()
        {
            ShowCancel(false);
            ShowMinimize(false);

            if (ApiInformation.IsApiContractPresent("Windows.ApplicationModel.FullTrustAppContract", 1, 0))
            {
                switch (Clockwork.GetTimer().action)
                {
                    case "shutdown":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("ShutdownGroup");
                        break;

                    case "restart":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("RestartGroup");
                        break;

                    case "hibernate":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("HibernateGroup");
                        break;

                    case "sleep":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("SleepGroup");
                        break;

                    case "logout":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("LogoutGroup");
                        break;

                    case "lock":
                        await FullTrustProcessLauncher.LaunchFullTrustProcessForCurrentAppAsync("LockGroup");
                        break;
                }
            }
            else
            {
                await DesktopExtensionDialogService.ShowDialogAsync();
            }
        }

        private void cancelButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            ShowCancel(false);
            ShowMinimize(false);
            timerClock.Stop();
        }

        private void editButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            ShowCancel(false);
            ShowMinimize(false);
            timerClock.Stop();
            this.Frame.Navigate(typeof(SetTimerPage));
        }

        private async void minimizeButton_Click(object sender, Windows.UI.Xaml.RoutedEventArgs e)
        {
            IList<AppDiagnosticInfo> infos = await AppDiagnosticInfo.RequestInfoForAppAsync();
            IList<AppResourceGroupInfo> resourceInfos = infos[0].GetResourceGroups();
            await resourceInfos[0].StartSuspendAsync();
        }
    }
}
